/*===========================================================================*/
/* Power Ten R-Series (R62,R63,R66,R82,& R83)                                */
/* LabWindows/CVI Instrument Driver                                          */
/* Original Release:                                                         */
/* By:  R. Poyner                                                            */
/* PH:  858-450-0085 x2431                                                   */
/* Fax: 858-678-4441                                                         */
/*                                                                           */
/* Modification History: None                                                */
/*===========================================================================*/

#include <visa.h>
#include <userint.h>
#include <ansi_c.h>
#include <formatio.h>
#include <string.h>

#include "PTRSeries.h"

#define PTRSeries_REVISION  "V1.00, 7/10/20002, CVI 6.0.0"  /* Driver Revision */
#define BUFFER_SIZE         128L                            /* File I/O buffer */

#define DEMO_IF			0
#define GPIB_IF			1
#define RS232_IF		2

#define LOCK                                                      \
    if ((PTRSeries_status = viLock(instrSession,				  \
                                  VI_EXCLUSIVE_LOCK,			  \
                                  THREAD_LOCK_TIMEOUT,			  \
                                  NULL,NULL)) < 0)                \
        return PTRSeries_status;                                  \

#define UNLOCK                                                    \
{                                                                 \
    if((unlockStatus = viUnlock(instrSession)) < 0)               \
        return unlockStatus;                                      \
    return PTRSeries_status;                                      \
}

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* PTRSeries_stringValPair is used in the PTRSeries_errorMessage function          */
/* PTRSeries_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  PTRSeries_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  PTRSeries_tStringValPair;
 

typedef struct PTRSeries_statusDataRanges *PTRSeries_instrRange;

static short int		user_if;  

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean PTRSeries_invalidViBooleanRange (ViBoolean val);
ViBoolean PTRSeries_invalidViInt16Range   (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean PTRSeries_invalidViInt32Range   (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean PTRSeries_invalidViUInt8Range   (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean PTRSeries_invalidViUInt16Range  (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean PTRSeries_invalidViUInt32Range  (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean PTRSeries_invalidViReal32Range  (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean PTRSeries_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus  PTRSeries_initCleanUp           (ViSession openRMSession,
                                          ViPSession openInstrSession,
                                          ViStatus currentStatus);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_init (ViRsrc     resourceName,
                                ViBoolean  IDQuery,
                                ViBoolean  resetDevice,
                                ViPSession instrSession)
{
    ViStatus	PTRSeries_status,unlockStatus;
    ViSession	rmSession = 0;
    ViUInt32	retCnt = 0;
    ViInt16		i;
    ViByte		rdBuffer[BUFFER_SIZE];
    ViByte		tmpBuffer[11];

    /*- Open instrument session ---------------------------------------------*/
    user_if = GPIB_IF;
    if ((PTRSeries_status = viOpenDefaultRM (&rmSession)) < 0)
    {
    	sprintf(rdBuffer,"viOpenDefaultRM - %08X",PTRSeries_status);
		MessagePopup("PTRSeries Error",rdBuffer);
        return PTRSeries_status;
    }

    if ((PTRSeries_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0)
    {
    	sprintf(rdBuffer,"viOpen - %08X",PTRSeries_status);
		MessagePopup("PTRSeries Error",rdBuffer);
        viClose (rmSession);
        return PTRSeries_status;
    }
	viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 3000);

	/* set up serial port if selected */
	if (strstr (resourceName, "ASRL") != NULL)
	{
		user_if = RS232_IF;
		viSetAttribute (*instrSession, VI_ATTR_ASRL_BAUD, 19200);
		viSetAttribute (*instrSession, VI_ATTR_ASRL_DATA_BITS, 8);
		viSetAttribute (*instrSession, VI_ATTR_ASRL_PARITY, VI_ASRL_PAR_NONE);
		viSetAttribute (*instrSession, VI_ATTR_ASRL_STOP_BITS, VI_ASRL_STOP_ONE);
		viSetAttribute (*instrSession, VI_ATTR_ASRL_FLOW_CNTRL, VI_ASRL_FLOW_NONE);
		viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_TERMCHAR);
		viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\r');
	}
	
    /*- Identification Query ------------------------------------------------*/
    if (IDQuery)
    {
		if ((PTRSeries_status = PTRSeries_idQuery (*instrSession,"1",rdBuffer)) < VI_SUCCESS)
            return PTRSeries_initCleanUp (rmSession, instrSession, PTRSeries_status);

		/* get and compare first eight bytes from *idn? response */
		for (i = 0; i < 9; ++i)
			tmpBuffer[i] = rdBuffer[i];
		tmpBuffer[9] = '\0';
		StringUpperCase (tmpBuffer);
		if (strcmp(tmpBuffer,"POWER TEN") != 0)
            return PTRSeries_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }
        
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice)
    {
        if ((PTRSeries_status = PTRSeries_reset (*instrSession, "1")) != VI_SUCCESS)
            return PTRSeries_initCleanUp (rmSession, instrSession, PTRSeries_status);
    }       
          
    return PTRSeries_status;
}

/*===========================================================================*/
/* Function: GenerateDCVolt                                                  */
/* Purpose:  This function sets the output voltage to the specified value.   */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_GenerateDCVolt (ViSession      instrSession,
                                          ViChar _VI_FAR channelName[],
                                          ViReal64       voltage)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

	LOCK

    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE ");
    sprintf(tmpBuffer,"%f",voltage);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

	UNLOCK
}

/*===========================================================================*/
/* Function: GenerateDCCurr                                                  */
/* Purpose:  This function sets the output current to the specified value.   */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_GenerateDCCurr (ViSession      instrSession,
                                          ViChar _VI_FAR channelName[],
                                          ViReal64       current)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

    LOCK
    
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT ");
    sprintf(tmpBuffer,"%f",current);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

	UNLOCK
}

/*===========================================================================*/
/* Function: GenerateDCVRamp                                                 */
/* Purpose:  This function initates a output voltage ramp from the present   */
/*           output voltage to the specified output voltage in the specified */
/*           time on the specified channel. Time is in seconds.              */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_GenerateDCVRamp  (ViSession      instrSession,
                                            ViChar _VI_FAR channelName[],
                                            ViReal64       voltage,
                                            ViReal64       time_sec)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

    LOCK
    
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE:RAMP ");
    sprintf(tmpBuffer,"%f ",voltage);
    strcat(cmdBuffer,tmpBuffer);
    sprintf(tmpBuffer,"%f",time_sec);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

	UNLOCK
}
                                          
/*===========================================================================*/
/* Function: GenerateDCCRamp                                                 */
/* Purpose:  This function initates a output Current ramp from the present   */
/*           output voltage to the specified output current in the specified */
/*           time on the specified channel. Time is in seconds.              */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_GenerateDCCRamp  (ViSession      instrSession,
                                            ViChar _VI_FAR channelName[],
                                            ViReal64       current,
                                            ViReal64       time_sec)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

    LOCK
    
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT:RAMP ");
    sprintf(tmpBuffer,"%f ",current);
    strcat(cmdBuffer,tmpBuffer);
    sprintf(tmpBuffer,"%f",time_sec);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

	UNLOCK
}

/*===========================================================================*/
/* Function: OutState                                                       */
/* Purpose:  This function reads the output voltage.                         */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_OutState (ViSession      instrSession,
                                    ViChar _VI_FAR channelName[],
                                    ViBoolean      state)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

    LOCK
    
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"OUTPUT");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":STATE ");
    sprintf(tmpBuffer,"%d",state);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

	UNLOCK
    
}

/*===========================================================================*/
/* Function: MeasureDCVolt                                                   */
/* Purpose:  This function reads the output voltage.                         */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_MeasureDCVolt (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViReal64       *voltage)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

    LOCK
    
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"MEASURE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query */
	if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *voltage = atof(rdBuffer);

	UNLOCK
}

/*===========================================================================*/
/* Function: MeasureDCCurr                                                   */
/* Purpose:  This function reads the output voltage.                         */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_MeasureDCCurr (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViReal64       *current)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

    LOCK
    
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"MEASURE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK
        
    /* get returned query */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *current = atof(rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: Mode                                                            */
/* Purpose:  This function returns the operating mode of the power supply.   */
/*           (voltage mode = 0, current mode = 1)                            */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ModeQuery (ViSession      instrSession,
                                     ViChar _VI_FAR channelName[],
                                     ViInt16        *mode)
{
    ViStatus		PTRSeries_status,unlockStatus;
	unsigned char	protCondReg  = 0;
	ViByte			statusMessage[BUFFER_SIZE];

	LOCK
	
	if ((PTRSeries_status = PTRSeries_ProtCondQuery(instrSession, channelName, &protCondReg, statusMessage)) < 0)
	{
		*mode = -1;
		UNLOCK;
	}

	if (protCondReg & PTRSeries_PE_CURRMODE)
		*mode = 1;
	else
		*mode = 0;
		
	UNLOCK
}

/*===========================================================================*/
/* Function: VoltMode                                                        */
/* Purpose:  This function configures the power supply to shutdown if the    */
/*           power supply switches from voltage mode to current mode.        */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_VoltMode (ViSession      instrSession,
                                    ViChar _VI_FAR channelName[],
                                    ViBoolean      serviceRequest)
                                
{
    ViStatus		PTRSeries_status,unlockStatus;
	unsigned char	mask;

	LOCK
	
	mask =  PTRSeries_PE_CURRMODE | PTRSeries_PE_FOLD;
	
	/* program foldback */
	if ((PTRSeries_status = PTRSeries_ConfigFoldback (instrSession, channelName, PTRSeries_FB_CURRMODE)) < 0)
		UNLOCK
		
	/* program protection event enable mask for foldback and current mode */
	if ((PTRSeries_status = PTRSeries_ConfigProtEventEnable (instrSession, channelName, mask)) < 0)
		UNLOCK
		
	if (serviceRequest)
	{
		/* program service request enable mask */
		PTRSeries_status = PTRSeries_ConfigSerReqEnable (instrSession, channelName, PTRSeries_SB_SRQ);
	}
	
	UNLOCK
}

/*===========================================================================*/
/* Function: CurrMode                                                        */
/* Purpose:  This function configures the power supply to shutdown if the    */
/*           power supply switches from current mode to voltage mode.        */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_CurrMode (ViSession      instrSession,
                                    ViChar _VI_FAR channelName[],
                                    ViBoolean      serviceRequest)
                                
{
    ViStatus	PTRSeries_status,unlockStatus;
	unsigned char	mask;

	LOCK
	
	mask =  PTRSeries_PE_VOLTMODE | PTRSeries_PE_FOLD;
	
	/* program foldback */
	if ((PTRSeries_status = PTRSeries_ConfigFoldback (instrSession, channelName, PTRSeries_FB_VOLTMODE)) < 0)
		UNLOCK
		
	/* program protection event enable mask */
	if ((PTRSeries_status = PTRSeries_ConfigProtEventEnable (instrSession, channelName, mask)) < 0)
		UNLOCK
		
	if (serviceRequest)
	{
		/* program service request enable mask */
		PTRSeries_status = PTRSeries_ConfigSerReqEnable (instrSession, channelName, PTRSeries_SB_SRQ);
	}
	
	UNLOCK    
}

/*===========================================================================*/
/* Function: NormalMode                                                      */
/* Purpose:  This function configures the power supply operate in both       */
/*           current mode and voltage mode. All protection events and        */
/*           service request enables are cleared.                            */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_NormalMode (ViSession      instrSession,
                                      ViChar _VI_FAR channelName[])
{
    ViStatus	PTRSeries_status,unlockStatus;

	LOCK
	
	/* program foldback */
	if ((PTRSeries_status = PTRSeries_ConfigFoldback (instrSession, channelName, PTRSeries_FB_NORMAL)) < 0)
		UNLOCK
		
	/* program protection event enable mask */
	if ((PTRSeries_status = PTRSeries_ConfigProtEventEnable (instrSession, channelName, 0)) < 0)
		UNLOCK
		
	/* program service request enable mask */
	PTRSeries_status = PTRSeries_ConfigSerReqEnable (instrSession, channelName, 0);

	UNLOCK    
}

/*===========================================================================*/
/* Function: IsolationRelay                                                  */
/* Purpose:  This function turns on/off the m9 external user connector       */
/*           isolation relay.                                                 */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_IsolationRelay (ViSession      instrSession,
                                          ViChar _VI_FAR channelName[],
                                          ViBoolean      state)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];
    
	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"OUTPUT");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":ISOLATION ");
    sprintf(tmpBuffer,"%d",state);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);
    
    UNLOCK
}

/*===========================================================================*/
/* Function: SenseRelay                                                      */
/* Purpose:  This function turns on/off the m9 external user connector sense */
/*           relay.															 */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_SenseRelay (ViSession      instrSession,
                                      ViChar _VI_FAR channelName[],
                                      ViBoolean      state)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];
    
	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"OUTPUT");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":SENSE ");
    sprintf(tmpBuffer,"%d",state);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);
    
    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigDCVolt                                                    */
/* Purpose:  This function configures the power supply for a triggered       */
/*           output voltage of the specified value. A software trigggger     */
/*           must be sent to the power supply to implement this sequence.    */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigDCVolt (ViSession      instrSession,
                                        ViChar _VI_FAR channelName[],
                                        ViReal64       voltage)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];
    
	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE:TRIGGERED ");
    sprintf(tmpBuffer,"%f",voltage);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}
                                          
/*===========================================================================*/
/* Function: ConfigDCCurr                                                    */
/* Purpose:  This function configures the power supply for a triggered       */
/*           output voltage of the specified value. A software trigggger     */
/*           must be sent to the power supply to implement this sequence.    */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigDCCurr (ViSession      instrSession,
                                        ViChar _VI_FAR channelName[],
                                        ViReal64       current)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];
    
	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT:TRIGGERED ");
    sprintf(tmpBuffer,"%f",current);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigDCVRamp                                                   */
/* Purpose:  This function configures the power supply for a triggered       */
/*           output voltage ramp from the present output voltage to the      */
/*           specified voltage in the specified time on the specified        */
/*           channel. A software trigger must be sent to the power supply to */
/*           implement this sequence.                                        */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigDCVRamp (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViReal64       voltage,
                                         ViReal64       time_sec)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];
    
	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE:RAMP:TRIGGERED ");
    sprintf(tmpBuffer,"%f ",voltage);
    strcat(cmdBuffer,tmpBuffer);
    sprintf(tmpBuffer,"%f",time_sec);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}
                                          
/*===========================================================================*/
/* Function: ConfigDCCRamp                                                   */
/* Purpose:  This function configures the power supply for a triggered       */
/*           output current ramp from the present output current to the      */
/*           specified current in the specified time on the specified        */
/*           channel. A software trigger must be sent to the power supply to */
/*           implement this sequence. Time is in seconds.                    */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigDCCRamp (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViReal64       current,
                                         ViReal64       time_sec)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];
    
	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT:RAMP:TRIGGERED ");
    sprintf(tmpBuffer,"%f ",current);
    strcat(cmdBuffer,tmpBuffer);
    sprintf(tmpBuffer,"%f",time_sec);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigOVP                                                       */
/* Purpose:  This function sets the output overvoltage protection to the     */
/*           specified value.                                                */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigOVP (ViSession      instrSession,
                                     ViChar _VI_FAR channelName[],
                                     ViReal64       voltage)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];
    
	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE:PROTECTION ");
    sprintf(tmpBuffer,"%f",voltage);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);

    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigSoftLimits                                                */
/* Purpose:  This function sets the soft voltage and current programming     */
/*           limits to the specified values.                                 */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigSoftLimits (ViSession      instrSession,
                                            ViChar _VI_FAR channelName[],
                                            ViReal64       voltage,
                                            ViReal64       current)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];
    
	LOCK
	
    /* build and send SCPI command string - voltage */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE:LIMIT ");
    sprintf(tmpBuffer,"%f",voltage);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* build and send SCPI command string - current */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT:LIMIT ");
    sprintf(tmpBuffer,"%f",current);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigPowerON                                                   */
/* Purpose:  This function configures the output power on values for the     */
/*           specified channel.                                              */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigPowerOn (ViSession		instrSession,
                                         ViChar _VI_FAR	channelName[],
                                         ViReal64		voltage,
                                         ViReal64		current,
                                         ViReal64		ovp)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

	LOCK
	
    /* build and send SCPI command string - voltage */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":INITIAL:VOLTAGE ");
    sprintf(tmpBuffer,"%f",voltage);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);

    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* build and send SCPI command string - current */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":INITIAL:CURRENT ");
    sprintf(tmpBuffer,"%f",current);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);

    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK
        
    /* build and send SCPI command string - ovp */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":INITIAL:VOLTAGE:PROTECTION ");
    sprintf(tmpBuffer,"%f",ovp);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);

    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* build and send SCPI command string - unlock */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":UNLOCK \"6867\"");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* build  and send SCPI command string - store */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":STORE");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* build and send SCPI command string - lock */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":LOCK");
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigFoldback                                                  */
/* Purpose:  This function configures the foldback mode if any. Three modes  */
/*           are normal, constant current, and constant voltage.             */
/*           See header file for bit definitions.                            */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigFoldback (ViSession      instrSession,
                                          ViChar _VI_FAR channelName[],
                                          ViInt16        mode)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"OUTPUT");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":PROTECTION:FOLD ");
    sprintf(tmpBuffer,"%d",mode);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigVoltAve                                                   */
/* Purpose:  This function configures the number of current measurents to be */
/*           averaged. 1-5                                                   */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigVoltAve (ViSession instrSession,
                                         ViChar    channelName[],
                                         ViInt16   average)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"MEASURE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE:AVERAGE ");
    sprintf(tmpBuffer,"%d",average);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigCurrAve                                                   */
/* Purpose:  This function configures the number of current measurents to be */
/*           averaged. 1-5                                                   */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigCurrAve (ViSession instrSession,
                                         ViChar    channelName[],
                                         ViInt16   average)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"MEASURE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT:AVERAGE ");
    sprintf(tmpBuffer,"%d",average);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigProtEventEnable                                           */
/* Purpose:  This function configures the protection event status enable     */
/*           register. See header file for bit definitions.                  */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigProtEventEnable (ViSession      instrSession,
                                                 ViChar _VI_FAR channelName[],
                                                 unsigned char  protectionEventEnable)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"STATUS");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":PROTECTION:ENABLE ");
    sprintf(tmpBuffer,"%d",protectionEventEnable);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigStandEventEnable                                          */
/* Purpose:  This function configures the standard event status enable       */
/*           register. See header file for bit definitions.                  */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigStandEventEnable (ViSession      instrSession,
											      ViChar _VI_FAR channelName[],
                                                  unsigned char  standardEventEnable)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"*ESE");
    strcat(cmdBuffer,channelName);
    sprintf(tmpBuffer," %d",standardEventEnable);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: ConfigSerReqEnable                                              */
/* Purpose:  This function configures the service request enable             */
/*           register. See header file for bit definitions.                  */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ConfigSerReqEnable (ViSession      instrSession,
                                              ViChar _VI_FAR channelName[],
                                              unsigned char  serviceRequestEnable)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"*SRE");
    strcat(cmdBuffer,channelName);
    sprintf(tmpBuffer," %d",serviceRequestEnable);
    strcat(cmdBuffer,tmpBuffer);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: OutputVoltQuery                                                */
/* Purpose:  This function returns the programmed output voltage value.      */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_OutputVoltQuery  (ViSession      instrSession,
                                            ViChar _VI_FAR channelName[],
                                            ViReal64       *voltage)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK
        
    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *voltage = atof(rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: OutputCurrQuery                                                */
/* Purpose:  This function returns the programmed output voltage value.      */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_OutputCurrQuery  (ViSession      instrSession,
                                            ViChar _VI_FAR channelName[],
                                            ViReal64       *current)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

	/* get returnd query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *current = atof(rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: OutputOVPQuery                                                  */
/* Purpose:  This function returns the programmed output voltage value.      */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_OutputOVPQuery (ViSession      instrSession,
                                           ViChar _VI_FAR channelName[],
                                           ViReal64       *OVP)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    /* build and send SCPI command string */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE:PROTECTION?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get return query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *OVP = atof(rdBuffer);

    UNLOCK
}
                                        

/*===========================================================================*/
/* Function: OutStateQuery                                                   */
/* Purpose:  This function returns the output state, 1 = ON.                 */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_OutStateQuery (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViBoolean      *state)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    /* build and send SCPI command string - voltage limit */
    strcpy(cmdBuffer,"OUTPUT");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":STATE?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK
        
    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *state = atoi(rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: IsoRelayQuery                                                   */
/* Purpose:  This function returns isolation relay state.                    */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_IsoRelayQuery (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViBoolean      *state)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    /* build and send SCPI command string - voltage limit */
    strcpy(cmdBuffer,"OUTPUT");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":ISOLATION?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK
        
    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *state = atoi(rdBuffer);

    UNLOCK
    
}

/*===========================================================================*/
/* Function: IsoRelayQuery                                                   */
/* Purpose:  This function returns sense relay state.                        */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_SenseRelayQuery (ViSession      instrSession,
                                           ViChar _VI_FAR channelName[],
                                           ViBoolean      *state)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    /* build and send SCPI command string - voltage limit */
    strcpy(cmdBuffer,"OUTPUT");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":SENSE?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK
        
    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *state = atoi(rdBuffer);

    UNLOCK
    
}

/*===========================================================================*/
/* Function: SoftLimitsQuery                                                 */
/* Purpose:  This function returns the voltage and current soft limits.      */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_SoftLimitsQuery  (ViSession      instrSession,
                                            ViChar _VI_FAR channelName[],
                                            ViReal64       *voltage,
                                            ViReal64       *current)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    /* build and send SCPI command string - voltage limit */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE:LIMIT?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK
        
    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *voltage = atof(rdBuffer);

	/* build and send SCPI command string - current limit */
    strcpy(cmdBuffer,"SOURCE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT:LIMIT?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *current = atof(rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: PowerONQuery                                                    */
/* Purpose:  This function returns the output power on values for the        */
/*           specified channel.                                              */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_PowerOnQuery  (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViReal64       *voltage,
                                         ViReal64       *current,
                                         ViReal64       *ovp)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];
    
	LOCK
	
	/* build and send SCPI command string - power on voltage */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":INITIAL:VOLTAGE?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *voltage = atof(rdBuffer);

	/* build and send SCPI comand string - power on current */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":INITIAL:CURRENT?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *current = atof(rdBuffer);

	/* build and send SCPI command string - power on ovp */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":INITIAL:VOLTAGE:PROTECTION?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *ovp = atof(rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: measVoltAveQuery                                                */
/* Purpose:  This function returns the number of voltage readings that will  */
/*           be averaged upon receipt of a voltage measurent command for the */
/*           specified channel.                                              */
/*===========================================================================*/
ViStatus PTRSeries_measVoltAveQuery (ViSession instrSession,
                                    ViChar    channelName[],
                                    ViInt16   *average)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    /* build and send SCPI command string - voltage limit */
    strcpy(cmdBuffer,"MEASURE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":VOLTAGE:AVERAGE?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK
        
    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *average = atoi(rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: measCurrAveQuery                                                */
/* Purpose:  This function returns the number of current readings that will  */
/*           be averaged upon receipt of a current measurent command for the */
/*           specified channel.                                              */
/*===========================================================================*/
ViStatus PTRSeries_measCurrAveQuery (ViSession instrSession,
                                    ViChar    channelName[],
                                    ViInt16   *average)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    /* build and send SCPI command string - voltage limit */
    strcpy(cmdBuffer,"MEASURE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":CURRENT:AVERAGE?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK
        
    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *average = atoi(rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: SendSWTrigger                                                   */
/* Purpose:  This function sends a software trigger to the specified         */
/*           channel. Setting the channel to zero causes a Group Execute     */
/*           Trigger for all power supplies that have been previously        */
/*           configured for triggered outputs.                               */
/*           Trigger Types: 1 = Voltage, 2 = Current, 3 = Both, 4 = Ramp     */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_SendSWTrigger (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViInt16        type)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		tmpBuffer[80];

	LOCK
	
	/* test for voltage, current, or both trigger type */
    if ((type > 0) && (type < 4))
    {
		/* build SCPI command string */
	    strcpy(cmdBuffer,"TRIGGER");
    	strcat(cmdBuffer,channelName);
	    strcat(cmdBuffer,":TYPE ");
	    sprintf(tmpBuffer,"%d",type);
	    strcat(cmdBuffer,tmpBuffer);
	}
	/* test for ramp trigger type */
	else if (type == 4)
    {
		/* build SCPI command string */
	    strcpy(cmdBuffer,"TRIGGER");
    	strcat(cmdBuffer,channelName);
	    strcat(cmdBuffer,":RAMP");
	}
	else
		return VI_ERROR_INV_PARAMETER;
	
	/* send SCPI command string */
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    UNLOCK
}

/*===========================================================================*/
/* Function: Abort Trigger                                                   */
/* Purpose:  This function clears all previously configured voltage and      */
/*           current triggers for the specified channel.                     */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_AbortTrigger (ViSession      instrSession,
                                        ViChar _VI_FAR channelName[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];

	LOCK
	
	/* build and send SCPI command string */
    strcpy(cmdBuffer,"TRIGGER");
   	strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":ABORT");
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: StatusByteQuery                                                 */
/* Purpose:  This function returns the instruments status byte and a message */
/*           indicating which bits are active.                               */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_StatusByteQuery (ViSession      instrSession,
                                           ViChar _VI_FAR channelName[],
                                           unsigned char  *statusByte,
                                           ViChar _VI_FAR statusMessage[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViChar		rdBuffer[BUFFER_SIZE];
    char		*endPtr;

	LOCK
	
	/* build and send SCPI command string */
    strcpy(cmdBuffer,"*STB");
   	strcat(cmdBuffer,channelName);
   	strcat(cmdBuffer,"?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

	/* assign value */
	*statusByte = (unsigned char) strtoul((const char *) rdBuffer, &endPtr, 10);

	/* generate status message from status byte bits, bits 0,3,7 are NOT USED */
	if (*statusByte == 0)
		strcpy(statusMessage,"00");
	else
	{
		sprintf(rdBuffer,"%02X - ",*statusByte);
		if (*statusByte & PTRSeries_SB_PROTEVENT)
			strcat(rdBuffer,"Protection Event, ");
		if (*statusByte & PTRSeries_SB_ERRORQUERY)
			strcat(rdBuffer,"Error Event Message, ");
		if (*statusByte & PTRSeries_SB_MAV)
			strcat(rdBuffer,"Message Available, ");
		if (*statusByte & PTRSeries_SB_STANDEVENT)
			strcat(rdBuffer,"Event Status, ");
		if (*statusByte & PTRSeries_SB_SRQ)
			strcat(rdBuffer,"Request Service");
		
		/* remove any ending ',' */
		retCnt = strlen(rdBuffer);
		if (rdBuffer[retCnt - 2] == ',')
			rdBuffer[retCnt - 2] = '\0';

		strcpy(statusMessage,rdBuffer);
	}
	
    UNLOCK
}

/*===========================================================================*/
/* Function: StandEventQuery                                                 */
/* Purpose:  This function returns the instruments standard event status     */
/*           register and a message indicating which bits are active.        */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_StandEventQuery (ViSession      instrSession,
                                           ViChar _VI_FAR channelName[],
                                           unsigned char  *standardEventStatusRegister,
                                           ViChar _VI_FAR statusMessage[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViChar		rdBuffer[BUFFER_SIZE];
    char		*endPtr;

	LOCK
	
	/* build and send SCPI command string */
    strcpy(cmdBuffer,"*ESR");
   	strcat(cmdBuffer,channelName);
   	strcat(cmdBuffer,"?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

	/* assign value */
	*standardEventStatusRegister = (unsigned char) strtoul((const char *) rdBuffer, &endPtr, 10);

	/* generate status message from register bits, bits 1,6 are NOT USED */
	if (*standardEventStatusRegister == 0)
		strcpy(statusMessage,"00");
	else
	{
		sprintf(rdBuffer,"%02X - ",*standardEventStatusRegister);
		if (*standardEventStatusRegister & PTRSeries_SE_OPC)
			strcat(rdBuffer,"Operation Complete, ");
		if (*standardEventStatusRegister & PTRSeries_SE_ERRORQUERY)
			strcat(rdBuffer,"Query Error, ");
		if (*standardEventStatusRegister & PTRSeries_SE_DEVICEERROR)
			strcat(rdBuffer,"Device Error, ");
		if (*standardEventStatusRegister & PTRSeries_SE_RANGEERROR)
			strcat(rdBuffer,"Execution Error (Range), ");
		if (*standardEventStatusRegister & PTRSeries_SE_SYNTAXERROR)
			strcat(rdBuffer,"Command Error (Syntax), ");
		if (*standardEventStatusRegister & PTRSeries_SE_PWRON)
			strcat(rdBuffer,"Power On");
		
		/* remove any ending ',' */
		retCnt = strlen(rdBuffer);
		if (rdBuffer[retCnt - 2] == ',')
			rdBuffer[retCnt - 2] = '\0';

		strcpy(statusMessage,rdBuffer);
	}
	
    UNLOCK
}

/*===========================================================================*/
/* Function: StandEventEnableQuery                                           */
/* Purpose:  This function returns the instruments standard event enable     */
/*           register and a message indicating which bits are active.        */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_StandEventEnableQuery (ViSession      instrSession,
                                                 ViChar _VI_FAR channelName[],
                                                 unsigned char  *standardEventEnableRegister,
                                                 ViChar _VI_FAR statusMessage[])
{
    
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViChar		rdBuffer[BUFFER_SIZE];
    char		*endPtr;

	LOCK
	
	/* build and send SCPI command string */
    strcpy(cmdBuffer,"*ESE");
   	strcat(cmdBuffer,channelName);
   	strcat(cmdBuffer,"?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

	/* assign value */
	*standardEventEnableRegister = (unsigned char) strtoul((const char *) rdBuffer, &endPtr, 10);

	/* generate status message from register bits, bits 1,6 are NOT USED */
	if (*standardEventEnableRegister == 0)
		strcpy(statusMessage,"00");
	else
	{
		sprintf(rdBuffer,"%02X - ",*standardEventEnableRegister);
		if (*standardEventEnableRegister & PTRSeries_SE_OPC)
			strcat(rdBuffer,"Operation Complete, ");
		if (*standardEventEnableRegister & PTRSeries_SE_ERRORQUERY)
			strcat(rdBuffer,"Query Error, ");
		if (*standardEventEnableRegister & PTRSeries_SE_DEVICEERROR)
			strcat(rdBuffer,"Device Error, ");
		if (*standardEventEnableRegister & PTRSeries_SE_RANGEERROR)
			strcat(rdBuffer,"Execution Error (Range), ");
		if (*standardEventEnableRegister & PTRSeries_SE_SYNTAXERROR)
			strcat(rdBuffer,"Command Error (Syntax), ");
		if (*standardEventEnableRegister & PTRSeries_SE_PWRON)
			strcat(rdBuffer,"Power On");
		
		/* remove any ending ',' */
		retCnt = strlen(rdBuffer);
		if (rdBuffer[retCnt - 2] == ',')
			rdBuffer[retCnt - 2] = '\0';

		strcpy(statusMessage,rdBuffer);
	}
	
    UNLOCK
}

/*===========================================================================*/
/* Function: ProtEventQuery                                                  */
/* Purpose:  This function returns the instruments protection event status   */
/*           register and a message indicating which bits are active.        */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ProtEventQuery (ViSession      instrSession,
                                          ViChar _VI_FAR channelName[],
                                          unsigned char  *protEventReg,
                                          ViChar _VI_FAR statusMessage[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViChar		rdBuffer[BUFFER_SIZE];
    char		*endPtr;

	LOCK
	
	/* build and send SCPI command string */
    strcpy(cmdBuffer,"STATUS");
   	strcat(cmdBuffer,channelName);
   	strcat(cmdBuffer,":PROTECTION:EVENT?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

	/* assign value */
	*protEventReg = (unsigned char) strtoul((const char *) rdBuffer, &endPtr, 10);

	/* generate status message from register bits */
	if (*protEventReg == 0)
		strcpy(statusMessage,"00");
	else
	{
		sprintf(rdBuffer,"%02X - ",*protEventReg);
		if (*protEventReg & PTRSeries_PE_VOLTMODE)
			strcat(rdBuffer,"Constant Voltage, ");
		if (*protEventReg & PTRSeries_PE_CURRMODE)
			strcat(rdBuffer,"Constant Current, ");
		if (*protEventReg & PTRSeries_PE_HW)
			strcat(rdBuffer,"Hardware Fault, ");
		if (*protEventReg & PTRSeries_PE_OVP)
			strcat(rdBuffer,"OVP, ");
		if (*protEventReg & PTRSeries_PE_OTP)
			strcat(rdBuffer,"OTP, ");
		if (*protEventReg & PTRSeries_PE_EXTSD)
			strcat(rdBuffer,"Ext. Shutdown, ");
		if (*protEventReg & PTRSeries_PE_FOLD)
			strcat(rdBuffer,"Foldback Mode, ");
		if (*protEventReg & PTRSeries_PE_PRGMERROR)
			strcat(rdBuffer,"Programming Error");
		
		/* remove any ending ',' */
		retCnt = strlen(rdBuffer);
		if (rdBuffer[retCnt - 2] == ',')
			rdBuffer[retCnt - 2] = '\0';

		strcpy(statusMessage,rdBuffer);
	}
	
    UNLOCK
}

/*===========================================================================*/
/* Function: ProtEventEnableQuery                                            */
/* Purpose:  This function returns the instruments protection event enable   */
/*           register and a message indicating which bits are active.        */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ProtEventEnableQuery (ViSession      instrSession,
                                                ViChar _VI_FAR channelName[],
                                                unsigned char  *protEventEnableRegister,
                                                ViChar _VI_FAR statusMessage[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViChar		rdBuffer[BUFFER_SIZE];
    char		*endPtr;

	LOCK
	
	/* build and send SCPI command string */
    strcpy(cmdBuffer,"STATUS");
   	strcat(cmdBuffer,channelName);
   	strcat(cmdBuffer,":PROTECTION:ENABLE?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

	/* assign value */
	*protEventEnableRegister = (unsigned char) strtoul((const char *) rdBuffer, &endPtr, 10);

	/* generate status message from register bits */
	if (*protEventEnableRegister == 0)
		strcpy(statusMessage,"00");
	else
	{
		sprintf(rdBuffer,"%02X - ",*protEventEnableRegister);
		if (*protEventEnableRegister & PTRSeries_PE_VOLTMODE)
			strcat(rdBuffer,"Constant Voltage, ");
		if (*protEventEnableRegister & PTRSeries_PE_CURRMODE)
			strcat(rdBuffer,"Constant Current, ");
		if (*protEventEnableRegister & PTRSeries_PE_HW)
			strcat(rdBuffer,"Hardware Fault, ");
		if (*protEventEnableRegister & PTRSeries_PE_OVP)
			strcat(rdBuffer,"OVP, ");
		if (*protEventEnableRegister & PTRSeries_PE_OTP)
			strcat(rdBuffer,"OTP, ");
		if (*protEventEnableRegister & PTRSeries_PE_EXTSD)
			strcat(rdBuffer,"Ext. Shutdown, ");
		if (*protEventEnableRegister & PTRSeries_PE_FOLD)
			strcat(rdBuffer,"Foldback Mode, ");
		if (*protEventEnableRegister & PTRSeries_PE_PRGMERROR)
			strcat(rdBuffer,"Programming Error");
		
		/* remove any ending ',' */
		retCnt = strlen(rdBuffer);
		if (rdBuffer[retCnt - 2] == ',')
			rdBuffer[retCnt - 2] = '\0';

		strcpy(statusMessage,rdBuffer);
	}
	
    UNLOCK
}

/*===========================================================================*/
/* Function: ProtCondQuery                                                   */
/* Purpose:  This function returns the instruments protection condition      */
/*           status register and a message indicating which bits are active. */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ProtCondQuery (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         unsigned char  *protCondReg,
                                         ViChar _VI_FAR statusMessage[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViChar		rdBuffer[BUFFER_SIZE];
    char		*endPtr;

	LOCK
	
	/* build and send SCPI command string */
    strcpy(cmdBuffer,"STATUS");
   	strcat(cmdBuffer,channelName);
   	strcat(cmdBuffer,":PROTECTION:CONDITION?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

	/* assign value */
	*protCondReg = (unsigned char) strtoul((const char *) rdBuffer, &endPtr, 10);

	/* generate status message from register bits */
	if (*protCondReg == 0)
		strcpy(statusMessage,"00");
	else
	{
		sprintf(rdBuffer,"%02X - ",*protCondReg);
		if (*protCondReg & PTRSeries_PE_VOLTMODE)
			strcat(rdBuffer,"Constant Voltage, ");
		if (*protCondReg & PTRSeries_PE_CURRMODE)
			strcat(rdBuffer,"Constant Current, ");
		if (*protCondReg & PTRSeries_PE_HW)
			strcat(rdBuffer,"Hardware Fault, ");
		if (*protCondReg & PTRSeries_PE_OVP)
			strcat(rdBuffer,"OVP, ");
		if (*protCondReg & PTRSeries_PE_OTP)
			strcat(rdBuffer,"OTP, ");
		if (*protCondReg & PTRSeries_PE_EXTSD)
			strcat(rdBuffer,"Ext. Shutdown, ");
		if (*protCondReg & PTRSeries_PE_FOLD)
			strcat(rdBuffer,"Foldback Mode, ");
		if (*protCondReg & PTRSeries_PE_PRGMERROR)
			strcat(rdBuffer,"Programming Error");
		
		/* remove any ending ',' */
		retCnt = strlen(rdBuffer);
		if (rdBuffer[retCnt - 2] == ',')
			rdBuffer[retCnt - 2] = '\0';

		strcpy(statusMessage,rdBuffer);
	}
	
    UNLOCK
}

/*===========================================================================*/
/* Function: ServiceReqEnableQuery                                           */
/* Purpose:  This function returns the instruments service request enable    */
/*           register and a message indicating which bits are active.        */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ServiceReqEnableQuery (ViSession      instrSession,
                                                 ViChar _VI_FAR channelName[],
                                                 unsigned char  *serviceReqEnableRegister,
                                                 ViChar _VI_FAR statusMessage[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViChar		rdBuffer[BUFFER_SIZE];
    char		*endPtr;

	LOCK
	
	/* build and send SCPI command string */
    strcpy(cmdBuffer,"*SRE");
   	strcat(cmdBuffer,channelName);
   	strcat(cmdBuffer,"?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

	/* assign value */
	*serviceReqEnableRegister = (unsigned char) strtoul((const char *) rdBuffer, &endPtr, 10);

	/* generate status message from status byte bits, bits 0,3,7 are NOT USED */
	if (*serviceReqEnableRegister == 0)
		strcpy(statusMessage,"00");
	else
	{
		sprintf(rdBuffer,"%02X - ",*serviceReqEnableRegister);
		if (*serviceReqEnableRegister & 0x02)
			strcat(rdBuffer,"Protection Event, ");
		if (*serviceReqEnableRegister & 0x04)
			strcat(rdBuffer,"Error Event Message, ");
		if (*serviceReqEnableRegister & 0x10)
			strcat(rdBuffer,"Message Available, ");
		if (*serviceReqEnableRegister & 0x20)
			strcat(rdBuffer,"Event Status, ");
		if (*serviceReqEnableRegister & 0x40)
			strcat(rdBuffer,"Request Service");
		
		/* remove any ending ',' */
		retCnt = strlen(rdBuffer);
		if (rdBuffer[retCnt - 2] == ',')
			rdBuffer[retCnt - 2] = '\0';

		strcpy(statusMessage,rdBuffer);
	}
	
    UNLOCK
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument to a known state.           */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_reset (ViSession      instrSession,
                                 ViChar _VI_FAR channelName[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];

	LOCK
	
    /*  send SCPI reset command */
    strcpy(cmdBuffer,"*RST");
   	strcat(cmdBuffer,channelName);
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    UNLOCK
}

/*===========================================================================*/
/* Function: Clear                                                          */
/* Purpose:  This function clear the instrument error and status info.       */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_clear (ViSession      instrSession,
                                 ViChar _VI_FAR channelName[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];

	LOCK
	
	/* send SCPI clear command */
    strcpy(cmdBuffer,"*CLS");
   	strcat(cmdBuffer,channelName);
    sendCnt = strlen(cmdBuffer);
    PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt);

    UNLOCK
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_errorQuery (ViSession      instrSession,
                                      ViPInt32       errorCode,
                                      ViChar _VI_FAR errorMessage[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViUInt32	index;
    ViUInt32	i;
    ViStatus	PTRSeries_status,unlockStatus;
    ViChar		*cptr;
    ViChar		rdBuffer[BUFFER_SIZE];
    ViByte		cmdBuffer[80];

	LOCK
	
    /*  send SCPI reset command */
    strcpy(cmdBuffer,"SYSTEM:ERROR?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

	/* get returned query string */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

	/* get error code from string */
	sscanf(rdBuffer,"%d",errorCode);
	
	/* get error message, skip ", " in return string */
	cptr  = strchr(rdBuffer,',');
	cptr += 2;
	strcpy(errorMessage,cptr);

	UNLOCK
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_errorMessage (ViSession      instrSession,
                                        ViStatus       statusCode,
                                        ViChar _VI_FAR message[])
{
    ViStatus PTRSeries_status,unlockStatus;
    ViInt16 i;
    static PTRSeries_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,                "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,                   "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,               "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,             "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,               "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,                  "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,                  "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,                  "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,                  "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,                  "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,                  "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,                  "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,                  "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,               "ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,                "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,             "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,            "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        
        /* not used, example of user defined Status Code
        {PREFIX_ERROR_INVALID_CONFIGURATION,   "ERROR: Instrument configuration error"},
        */
        
        {VI_NULL, VI_NULL}
    };

    PTRSeries_status = viStatusDesc (instrSession, statusCode, message);
    if (PTRSeries_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == statusCode) {
                strcpy (message, statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        sprintf (message, "Unknown Error 0x%08lX", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    UNLOCK
}

/*===========================================================================*/
/* Function: revisionQuery                                                   */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_revisionQuery (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViChar _VI_FAR driverRev[],
                                         ViChar _VI_FAR instrRev[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViInt16		i;
    ViChar		*cptr;
    ViByte		rdBuffer[BUFFER_SIZE];
    ViByte		cmdBuffer[BUFFER_SIZE];

	LOCK
	
	/* send SCPI identify command */
	strcpy(cmdBuffer,"*IDN");
	strcat(cmdBuffer,channelName);
	strcat(cmdBuffer,"?");
	sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

    /* parse *idn? return string for firware revisions - 4 & 5 fields */
	cptr = rdBuffer;
	for (i = 0; i < 3; ++i)
	{
		cptr = strchr(cptr,',');
		++cptr;
	}
	
	/* copy driver and intrument strings to buffers */
	strcpy(instrRev, cptr);
    strcpy (driverRev, PTRSeries_REVISION);

	UNLOCK
}

/*===========================================================================*/
/* Function: Identify Query                                                  */
/* Purpose:  This function returns the *idn? response.                       */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_idQuery (ViSession      instrSession,
                                   ViChar _VI_FAR channelName[],
                                   ViChar _VI_FAR idBuffer[])
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViInt16		i;
    ViChar		*cptr;
    ViByte		rdBuffer[BUFFER_SIZE];
    ViByte		cmdBuffer[80];

	LOCK
	
	/* send SCPI identify command */
	strcpy(cmdBuffer,"*IDN");
	strcat(cmdBuffer,channelName);
	strcat(cmdBuffer,"?");
	sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';

	/* copy id string to buffer */
	strcpy(idBuffer, rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: ModelQuery                                                   */
/* Purpose:  This function reads the instrument's Maximum Voltage and        */
/*           Current for the specified channel.                              */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_ModelQuery    (ViSession      instrSession,
                                         ViChar _VI_FAR channelName[],
                                         ViReal64       *modelVolt,
                                         ViReal64       *modelCurr)
{
    ViUInt32	retCnt = 0;
    ViUInt32	sendCnt = 0;
    ViStatus	PTRSeries_status,unlockStatus;
    ViByte		cmdBuffer[80];
    ViByte		rdBuffer[BUFFER_SIZE];

	LOCK
	
    *modelVolt = 0.0;
    *modelCurr = 0.0;

    /* build and send SCPI command string - voltage */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":MODEL:VOLTAGE?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *modelVolt = atof(rdBuffer);

    /* build and send SCPI command string - current */
    strcpy(cmdBuffer,"CALIBRATE");
    strcat(cmdBuffer,channelName);
    strcat(cmdBuffer,":MODEL:CURRENT?");
    sendCnt = strlen(cmdBuffer);
    if ((PTRSeries_status = viWrite (instrSession, cmdBuffer, sendCnt, &retCnt)) < 0)
        UNLOCK

    /* get returned query */
    if ((PTRSeries_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
        UNLOCK
	rdBuffer[retCnt] = '\0';
    
    *modelCurr = atof(rdBuffer);

    UNLOCK
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus CVIFUNC PTRSeries_close (ViSession instrSession)
{
    PTRSeries_instrRange instrPtr;
    ViSession rmSession;
    ViStatus PTRSeries_status;

    if ((PTRSeries_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
		return PTRSeries_status;
 	if ((PTRSeries_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
		return PTRSeries_status;
            
    if (instrPtr != NULL) 
        free (instrPtr);
    
    PTRSeries_status = viClose (instrSession);
    viClose (rmSession);

	return PTRSeries_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean PTRSeries_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean PTRSeries_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean PTRSeries_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean PTRSeries_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean PTRSeries_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean PTRSeries_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean PTRSeries_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean PTRSeries_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the PTRSeries_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from PTRSeries_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus PTRSeries_initCleanUp (ViSession  openRMSession,
                               ViPSession openInstrSession,
                               ViStatus   currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
